/*---------------- LISTING 1  -------  CIRCLE.C ---------------------*
* File:       Circle.C                                               *
* Desc:       Implementation of Bresenham's Circle Algorithm         *
* Author:     Marv Luse, Ithaca Street Software, Inc.                *
* Date:       January, 1990                                          *
* Copyright:  Use freely, but acknowledge authorship                 *
* Notes:      SET_PIXEL should be set appropriately for the compiler *
*             and graphics package being used.                       *
* Compilers:  Turbo C 2.0 and Microsoft C 5.1                        *
* Switches:   #define these as needed                                *
*             BGI - for Borland's Graphics Interface                 *
*             MSC - for Microsoft's graphics routines                *
*             META - for MetaWINDOW's display routines               *
*                                                                    *
*             TEST - to get a test driver (Turbo C only)             *
* -------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#ifdef BGI
#include <graphics.h>
#define SET_PIXEL( x, y )  putpixel( x, y, 15 )
#endif

/* Microsoft C Graphics */
#ifdef MSC
#include <graph.h>
#define SET_PIXEL( x, y )  _setpixel( x, y )
#endif

/* Metagraphics MetaWINDOW */
#ifdef META
#define SET_PIXEL( x, y )  SetPixel( x, y )
#endif

/*--------------------------------------------------------------------*/
/* sets pixels symmetrically on circumference of circle               */

void Set8Pixels( int xc, int yc, int dx, int dy )
{
	SET_PIXEL( xc+dx, yc+dy );
	SET_PIXEL( xc+dx, yc-dy );
	SET_PIXEL( xc-dx, yc+dy );
	SET_PIXEL( xc-dx, yc-dy );
	if( dx != dy ) {
		SET_PIXEL( xc+dy, yc+dx );
		SET_PIXEL( xc+dy, yc-dx );
		SET_PIXEL( xc-dy, yc+dx );
		SET_PIXEL( xc-dy, yc-dx );
	}
}

/*--------------------------------------------------------------------*/
/* draw a circle of radius r centered on (xc,yc)                      */

void Circle( int xc, int yc, int r )
{
	int dx, dy, err;

	SET_PIXEL( xc, yc+r );
	SET_PIXEL( xc, yc-r );
	SET_PIXEL( xc+r, yc );
	SET_PIXEL( xc-r, yc );

	dx = 1;
	dy = r;
	err = 9 - (r << 1);

	while( dx <= dy ) {
		Set8Pixels( xc, yc, dx, dy );
		if( err > 0 ) {
			err += ((dx-dy) << 2) + 10;
			dy--;
		}
		else
			err += (dx << 2) + 6;
		dx++;
	}
}

#ifdef TEST

#ifndef BGI
#error Test driver is only available for Turbo C
#endif

/*--------------------------------------------------------------------*/
/* Function to initialize the Turbo C graphics environment            */

char *GrafPath = "E:\\TURBOC\\";   /* Change this as needed */
int   GrafDriver, GrafMode, GrafError;

void GrafInit ( void )
{
	GrafDriver = DETECT;
	GrafMode = 0;
	initgraph(&GrafDriver, &GrafMode, GrafPath);
	GrafError = graphresult();
	if ( GrafError != grOk ) {
        printf ("\nGraphics System Error: %s", 
                    grapherrormsg(GrafError));
		printf ("\nExecution Terminated");
		exit ( 8 );
	}
}

/*--------------------------------------------------------------------*/
/* Function to terminate the graphics environment                     */

void GrafTerm ( void )
{
	closegraph();
}

/*--------------------------------------------------------------------*/
/* Program to illustrate the circle drawing code                      */

void main ( void )
{
	int x, y, r;

	GrafInit();
	x = getmaxx() / 2;
	y = getmaxy() / 2;
	r = y - 50;

	Circle( x, y, r );
	Circle( x, y, r/2 );
	getch();

	GrafTerm();
}
#endif