/**** EMS2.C ****
	Part 2 of the treatment of Expanded Memory
	Part 1 by Michael Young in the Spring 1990 issue (4:3)

	(c) Copyright 1990 Andrew Binstock / The C Gazette
	Use freely if authorship is acknowledged.

	Verified for Turbo C and Microsoft
 ****************/
#define MSC 1             /* Leave for Microsoft, comment out for others */

#ifdef MSC                /* Be careful of MSC's defaults */
#pragma pack (1)
#endif

#define DRIVER 1          /* Define it to test/demo the EMM functions */

#include <stdio.h>
#include <dos.h>
#include <string.h>

#define FAR far           /* For portability */

struct emm_table {        /* Table of handles and their names */
	int open_handle;
	char open_name[8];
};

#include "ems2.h"         /* Do not omit this! */

#define GET_NAME 0
#define SET_NAME 1
#define SAVE     0x47
#define RESTORE  0x48
#define CONV_TO_EMS 0
#define EMS_TO_CONV 1

static union REGS inregs, outregs;
static struct SREGS segs;

int EmsNameHandle (char FAR *name, int handle){ /* Assign a name to a handle */

	segs.ds     = FP_SEG (name);
	inregs.x.si = FP_OFF (name);
	inregs.x.dx = handle;
	inregs.x.ax = 0x5301;
	int86x (0x67, &inregs, &outregs, &segs);

	return ((outregs.x.ax & 0xFF00) == 0 ? 0 : -1);
}

int EmsGetNamedHandle (char FAR * name) { /* Convert handle name to number  */
                                          /* Return handle #, or -1 on err. */
	segs.ds     = FP_SEG (name);
	inregs.x.si = FP_OFF (name);
	inregs.x.ax = 0x5401;
	int86x (0x67, &inregs, &outregs, &segs);

	return ((outregs.x.ax & 0xFF00) == 0 ? outregs.x.dx : -1);
}

int EmsGetTable (struct emm_table FAR * table){ /* Loads a table with the    */
                                                /* names of active handles   */
    segs.es     = FP_SEG (table);               /* Returns # of currently    */
    inregs.x.di = FP_OFF (table);               /* active handles.           */
	inregs.x.ax = 0x5400;
	int86x (0x67, &inregs, &outregs, &segs);
	return (outregs.x.ax & 0x00FF);
}

int EmsPreservePageMap (int SaveRestore,       /* Either saves or restores  */
                        int TsrHandle) {       /* page-mapping registers.   */
	if (!EmsInstalled() || !Allocated)         /* For use by TSR's, to pro- */
        return (-1);                           /* tect host application's   */
                                                           /* EMS state.  */
	if (SaveRestore != SAVE && SaveRestore != RESTORE)
		return (-1);

    inregs.x.dx = TsrHandle;
	inregs.h.ah = SaveRestore;
	int86 (0x67, &inregs, &outregs);

	return (outregs.h.ah != 0 ? -1 : 0);  /* AH = 0, if all went well */
}

int EmsMoveMem (int move_type, char FAR* buffer,
                int move_handle, int page, unsigned long length) {
	struct {
		unsigned long size;         /* number of bytes to move             */

		char source_mem_type;       /* source is: 0 conventional, 1 EMM    */
		unsigned source_handle;     /* 0 if conventional, handle # if EMM  */
		unsigned source_offset;     /* from seg if conv., into page if EMM */
		unsigned source_segment;    /* seg if conv., logical page if EMM   */

		char dest_mem_type;         /* duplicates source fields but for    */
		unsigned dest_handle;       /*    destination of move or exchange  */
		unsigned dest_offset;
		unsigned dest_segment;
	} ems_move_params, FAR *ems_move_params_ptr;

	ems_move_params.size = length;

	ems_move_params.source_mem_type = move_type;
	ems_move_params.source_handle  = (move_type == 0 ? 0 : move_handle);
	ems_move_params.source_offset  = (move_type == 0 ? FP_OFF (buffer) : 0 );
	ems_move_params.source_segment = (move_type == 0 ? FP_SEG (buffer) : page);

	ems_move_params.dest_mem_type  = (move_type == 0 ? 1 : 0);
	ems_move_params.dest_handle  = (move_type != 0 ? 0 : move_handle);
	ems_move_params.dest_offset  = (move_type != 0 ? FP_OFF (buffer) : 0 );
	ems_move_params.dest_segment = (move_type != 0 ? FP_SEG (buffer) : page);

	inregs.x.ax = 0x5700;
	ems_move_params_ptr = &ems_move_params;
	inregs.x.si = FP_OFF (ems_move_params_ptr);
	segs.ds = FP_SEG (ems_move_params_ptr);
	int86x (0x67, &inregs, &outregs, &segs);

	return (outregs.h.ah != 0 ? -1 : 0);                /* 0 means all OK */
}

#if DRIVER
struct emm_table table[255];    /* the table of handle info */

void main() {

	int i, active_handles;
	char handle_name [9];

	if (! EmsInstalled()) {         /* Always check for active EMM first */
		puts ("Error in finding/using EMM");
		return;
	}
	else                            /* Show how much free EMM is available */
        printf ("EMM found with %d unallocated pages\n", i = EmsPagesAvail());

    if (i > 2) {                          /* Get two pages */
		if (EmsAlloc (2) != NULL)
			printf ("Allocated 2 pages to handle %d\n", Handle);
	}
    else {  puts ("Error allocating memory");  return;   }

    EmsNameHandle ("GaZeTTE", Handle);    /* Give our handle a name */

	active_handles = EmsGetTable (table); /* Get table of all handle names  */

    for (i = 0; i < active_handles; i++){ /* Print table of handle names    */
		strncpy (handle_name, table[i].open_name, 8);
		handle_name[8] = '\0';
		printf("handle: %d name: %s\n", table[i].open_handle, handle_name);
	}

	EmsMoveMem (CONV_TO_EMS, (char FAR*)table, Handle, 0,    /* Copy table */
        active_handles * sizeof (struct emm_table));         /* to EMM     */

	memset (table, 'A',                        /* Write A's over table area */
		active_handles * sizeof (struct emm_table));

	printf ("\n");
    for (i = 0; i < active_handles; i++) {      /* Display garbage table */
		strncpy (handle_name, table[i].open_name, 8);
		handle_name[8] = '\0';
		printf("handle: %d name: %s\n", table[i].open_handle, handle_name);
	}

	EmsMoveMem (EMS_TO_CONV, (char FAR*) table, Handle, 0,   /* Restore the */
		active_handles * sizeof (struct emm_table)); /* EMM table   */

	printf ("\n");
    for (i = 0; i < active_handles; i++) {     /* Display restored table */
		strncpy (handle_name, table[i].open_name, 8);
		handle_name[8] = '\0';
		printf("handle: %d name: %s\n", i, handle_name);
	}

	i = EmsGetNamedHandle ("GaZeTTE");
	printf ("\n\'GaZeTTE\' assigned to handle %d\n", i);

	EmsFree();                                 /* Free up our allocated EMM */
}
#endif